@@ -1,4 +1,6 @@
-#!/usr/bin/awish
+#!/bin/sh
+# \
+exec tclsh "$0" ${1+"$@"}
 
 ##############################################################
 #
@@ -19,14 +21,9 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #
-# TCL/TK GUI for SE Linux policy analysis
-# Requires tcl and tk 8.4+, with BWidget
+# Question/comments to: setools@tresys.com
 #
-# Question/comments to: selinux@tresys.com
-#
-# This tools is designed to analysis selinux policies.
-#
-# See the assoicated help file for more information.
+# This tool is designed to analyze SELinux policies.  See the
+# assoicated help file for more information.
 #
 ##############################################################
-
