@@ -13,121 +13,63 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-# TCL/TK GUI for SELinux policy analysis
-# Requires tcl and tk 8.4+, with BWidget
-
-
-##############################################################
-# ::Apol_RBAC
-#
-# The Access Control Rules page
-##############################################################
 namespace eval Apol_RBAC {
     variable vals
     variable widgets
 }
 
-proc Apol_RBAC::search { str case_Insensitive regExpr srch_Direction } {
-    variable widgets
-    ApolTop::textSearch $widgets(results).tb $str $case_Insensitive $regExpr $srch_Direction
-}
-
-proc Apol_RBAC::set_Focus_to_Text {} {
-    focus $Apol_RBAC::widgets(results)
-}
-
-proc Apol_RBAC::searchRBACs {} {
+proc Apol_RBAC::create {tab_name nb} {
     variable vals
     variable widgets
 
-    Apol_Widget::clearSearchResults $widgets(results)
-    if {![ApolTop::is_policy_open]} {
-        tk_messageBox -icon error -type ok -title "Error" -message "No current policy file is opened!"
-        return
-    }
+    _initializeVars
 
-    switch -- $vals(rule_selection) {
-        "allow" { set rule_selection allow}
-        "trans" { set rule_selection role_transition}
-        "both"  { set rule_selection [list allow role_transition] }
-    }
-    set other_opts {}
-    set source_sym {}
-    if {$vals(source:use)} {
-        if {$vals(source:sym) == {}} {
-            tk_messageBox -icon error -type ok -title "Error" -message "No source role selected."
-            return
-        }
-        if {$vals(source:which) == "either"} {
-            lappend other_opts "source_any"
-        }
-        set source_sym $vals(source:sym)
-    }
-    set target_sym {}
-    if {$rule_selection == "allow" && $vals(target_role:use) && \
-            (!$vals(source:use) || $vals(source:which) != "either")} {
-        if {$vals(target_role:sym) == {}} {
-            tk_messageBox -icon error -type ok -title "Error" -message "No target role selected."
-            return
-        }
-        set target_sym $vals(target_role:sym)
-    } elseif {$rule_selection == "role_transition" && $vals(target_type:use)} {
-        if {$vals(target_type:sym) == {}} {
-            tk_messageBox -icon error -type ok -title "Error" -message "No target type selected."
-            return
-        }
-        set target_sym $vals(target_type:sym)
-    }
-    set default_sym {}
-    if {$rule_selection == "role_transition" && $vals(default:use) && \
-            (!$vals(source:use) || $vals(source:which) != "either")} {
-        if {$vals(default:sym) == {}} {
-            tk_messageBox -icon error -type ok -title "Error" -message "No default role selected."
-            return
-        }
-        set default_sym $vals(default:sym)
-    }
+    set frame [$nb insert end $tab_name -text "RBAC Rules"]
+    set topf [frame $frame.top]
+    set bottomf [frame $frame.bottom]
+    pack $topf -expand 0 -fill both -pady 2
+    pack $bottomf -expand 1 -fill both -pady 2
 
-    if {[catch {apol_SearchRBACRules $rule_selection $other_opts $source_sym $target_sym $default_sym} results]} {
-        tk_messageBox -icon error -type ok -title "Error" -message "Error searching RBAC rules:\n$results"
-        return
-    }
+    set rsbox [TitleFrame $topf.rs -text "Rule Selection"]
+    set obox [TitleFrame $topf.opts -text "Search Options"]
+    set dbox [TitleFrame $bottomf.results -text "RBAC Rules Display"]
+    pack $rsbox -side left -expand 0 -fill both -padx 2
+    pack $obox -side left -expand 1 -fill both -padx 2
+    pack $dbox -expand 1 -fill both -padx 2
 
-    if {[llength $results] == 0} {
-        set text "Search returned no results."
-    } else {
-        set text "[llength $results] rule"
-        if {[llength $results] != 1} {
-            append text s
-        }
-        append text " match the search criteria.\n\n"
-    }
-    Apol_Widget::appendSearchResultText $widgets(results) $text
-    foreach r [lsort $results] {
-        renderRBAC $r
-    }
+    # Rule selection subframe
+    set rs [$rsbox getframe]
+    radiobutton $rs.allow -text allow -value allow \
+        -variable Apol_RBAC::vals(rule_selection)
+    radiobutton $rs.trans -text role_transition -value trans \
+        -variable Apol_RBAC::vals(rule_selection)
+    radiobutton $rs.both -text "allow and role_transition" -value both \
+        -variable Apol_RBAC::vals(rule_selection)
+    trace add variable Apol_RBAC::vals(rule_selection) write \
+        [list Apol_RBAC::_ruleChanged]
+    pack $rs.allow $rs.trans $rs.both -side top -anchor w
+
+    set widgets(options_pm) [PagesManager [$obox getframe].opts]
+
+    _allowCreate [$widgets(options_pm) add allow]
+    _transCreate [$widgets(options_pm) add trans]
+    _bothCreate [$widgets(options_pm) add both]
+    trace add variable Apol_RBAC::vals(source:which) write Apol_RBAC::_toggleRoleBox
+
+    $widgets(options_pm) compute_size
+    pack $widgets(options_pm) -expand 1 -fill both -side left
+    $widgets(options_pm) raise allow
+
+    set ok [button [$obox getframe].ok -text OK -width 6 -command Apol_RBAC::_searchRBACs]
+    pack $ok -side right -padx 5 -pady 5 -anchor ne
+
+    set widgets(results) [Apol_Widget::makeSearchResults [$dbox getframe].results]
+    pack $widgets(results) -expand yes -fill both
+
+    return $frame
 }
 
-proc Apol_RBAC::renderRBAC {rule} {
-    variable widgets
-    foreach {rule_type source_set target_set default } $rule {
-        if {[llength $source_set] > 1} {
-            set source_set "\{ $source_set \}"
-        }
-        if {[llength $target_set] > 1} {
-            set target_set "\{ $target_set \}"
-        }
-        if {$default != {}} {
-            Apol_Widget::appendSearchResultLine $widgets(results) 0 \
-                {} $rule_type $source_set $target_set $default
-        } else {
-            Apol_Widget::appendSearchResultLine $widgets(results) 0 \
-                {} $rule_type $source_set $target_set
-        }
-    }
-}
-
-proc Apol_RBAC::open { } {
+proc Apol_RBAC::open {ppath} {
     variable vals
     variable widgets
     $widgets(allow:source) configure -values $Apol_Roles::role_list
@@ -143,10 +85,10 @@ proc Apol_RBAC::open { } {
     set vals(rule_selection) allow
 }
 
-proc Apol_RBAC::close { } {
+proc Apol_RBAC::close {} {
     variable widgets
 
-    initializeVars
+    _initializeVars
     $widgets(allow:source) configure -values {}
     $widgets(allow:target) configure -values {}
     $widgets(trans:source) configure -values {}
@@ -155,7 +97,14 @@ proc Apol_RBAC::close { } {
     $widgets(both:source) configure -values {}
 }
 
-proc Apol_RBAC::initializeVars {} {
+proc Apol_RBAC::getTextWidget {} {
+    variable widgets
+    return $widgets(results).tb
+}
+
+#### private functions below ####
+
+proc Apol_RBAC::_initializeVars {} {
     variable vals
     array set vals {
         rule_selection allow
@@ -176,74 +125,7 @@ proc Apol_RBAC::initializeVars {} {
     }
 }
 
-proc Apol_RBAC::goto_line { line_num } {
-    variable widgets
-    Apol_Widget::gotoLineSearchResults $widgets(results) $line_num
-}
-
-proc Apol_RBAC::create {nb} {
-    variable vals
-    variable widgets
-
-    initializeVars
-
-    # Layout frames
-    set frame [$nb insert end $ApolTop::rbac_tab -text "RBAC Rules"]
-    set topf [frame $frame.top]
-    set bottomf [frame $frame.bottom]
-    pack $topf -expand 0 -fill both -pady 2
-    pack $bottomf -expand 1 -fill both -pady 2
-
-    # Major subframes
-    set rsbox [TitleFrame $topf.rs -text "Rule Selection"]
-    set obox [TitleFrame $topf.opts -text "Search Options"]
-    set dbox [TitleFrame $bottomf.results -text "RBAC Rules Display"]
-    pack $rsbox -side left -expand 0 -fill both -padx 2
-    pack $obox -side left -expand 1 -fill both -padx 2
-    pack $dbox -expand 1 -fill both -padx 2
-
-    # Rule selection subframe
-    set rs [$rsbox getframe]
-    radiobutton $rs.allow -text allow -value allow \
-        -variable Apol_RBAC::vals(rule_selection)
-    radiobutton $rs.trans -text role_transition -value trans \
-        -variable Apol_RBAC::vals(rule_selection)
-    radiobutton $rs.both -text "allow and role_transition" -value both \
-        -variable Apol_RBAC::vals(rule_selection)
-    trace add variable Apol_RBAC::vals(rule_selection) write \
-        [list Apol_RBAC::ruleChanged]
-    pack $rs.allow $rs.trans $rs.both -side top -anchor w
-
-    # Search criteria subframe
-    set widgets(options_pm) [PagesManager [$obox getframe].opts]
-
-    allowCreate [$widgets(options_pm) add allow]
-    transCreate [$widgets(options_pm) add trans]
-    bothCreate [$widgets(options_pm) add both]
-    trace add variable Apol_RBAC::vals(source:which) write Apol_RBAC::toggleRoleBox
-
-    $widgets(options_pm) compute_size
-    pack $widgets(options_pm) -expand 1 -fill both -side left
-    $widgets(options_pm) raise allow
-
-    set ok [button [$obox getframe].ok -text OK -width 6 -command Apol_RBAC::searchRBACs]
-    pack $ok -side right -padx 5 -pady 5 -anchor ne
-
-    # Display results window
-    set widgets(results) [Apol_Widget::makeSearchResults [$dbox getframe].results]
-    pack $widgets(results) -expand yes -fill both
-
-    return $frame
-}
-
-proc Apol_RBAC::ruleChanged {name1 name2 ops} {
-    variable vals
-    variable widgets
-    Apol_Widget::clearSearchResults $widgets(results)
-    $widgets(options_pm) raise $vals(rule_selection)
-}
-
-proc Apol_RBAC::allowCreate {a_f} {
+proc Apol_RBAC::_allowCreate {a_f} {
     variable vals
     variable widgets
 
@@ -264,7 +146,7 @@ proc Apol_RBAC::allowCreate {a_f} {
                        -variable Apol_RBAC::vals(source:which) \
                        -value either]
     trace add variable Apol_RBAC::vals(source:use) write \
-        [list Apol_RBAC::toggleCheckbutton $widgets(allow:source) [list $which_source $which_any]]
+        [list Apol_RBAC::_toggleCheckbutton $widgets(allow:source) [list $which_source $which_any]]
     pack $which_source $which_any -side top -anchor w
     pack $source_cb -side top -anchor w
     pack $widgets(allow:source) -side top -expand 0 -fill x -padx 4
@@ -279,13 +161,13 @@ proc Apol_RBAC::allowCreate {a_f} {
                                    -textvariable Apol_RBAC::vals(target_role:sym) \
                                    -helptext "Type or select a role" -autopost 1]
     trace add variable Apol_RBAC::vals(target_role:use) write \
-        [list Apol_RBAC::toggleCheckbutton $widgets(allow:target) {}]
+        [list Apol_RBAC::_toggleCheckbutton $widgets(allow:target) {}]
     pack $widgets(allow:target_cb) -side top -anchor w
     pack $widgets(allow:target) -side top -expand 0 -fill x -padx 4
     pack $target -side left -padx 4 -pady 2 -expand 0 -fill y
 }
 
-proc Apol_RBAC::transCreate {t_f} {
+proc Apol_RBAC::_transCreate {t_f} {
     variable vals
     variable widgets
 
@@ -306,7 +188,7 @@ proc Apol_RBAC::transCreate {t_f} {
                        -variable Apol_RBAC::vals(source:which) \
                        -value either]
     trace add variable Apol_RBAC::vals(source:use) write \
-        [list Apol_RBAC::toggleCheckbutton $widgets(trans:source) [list $which_source $which_any]]
+        [list Apol_RBAC::_toggleCheckbutton $widgets(trans:source) [list $which_source $which_any]]
     pack $which_source $which_any -side top -anchor w
     pack $source_cb -side top -anchor w
     pack $widgets(trans:source) -side top -expand 0 -fill x -padx 4
@@ -325,15 +207,15 @@ proc Apol_RBAC::transCreate {t_f} {
                    -variable Apol_RBAC::vals(target_type:types)]
     set attribs [checkbutton $ta_frame.attribs -text "Attribs" -state disabled \
                    -variable Apol_RBAC::vals(target_type:attribs)]
-    $types configure -command [list Apol_RBAC::toggleTAPushed $types]
-    $attribs configure -command [list Apol_RBAC::toggleTAPushed $attribs]
+    $types configure -command [list Apol_RBAC::_toggleTAPushed $types]
+    $attribs configure -command [list Apol_RBAC::_toggleTAPushed $attribs]
     trace add variable Apol_RBAC::vals(target_type:types) write \
-        [list Apol_RBAC::toggleTASym]
+        [list Apol_RBAC::_toggleTASym]
     trace add variable Apol_RBAC::vals(target_type:attribs) write \
-        [list Apol_RBAC::toggleTASym]
+        [list Apol_RBAC::_toggleTASym]
     pack $types $attribs -side left -padx 2
     trace add variable Apol_RBAC::vals(target_type:use) write \
-        [list Apol_RBAC::toggleCheckbutton $widgets(trans:target) [list $types $attribs]]
+        [list Apol_RBAC::_toggleCheckbutton $widgets(trans:target) [list $types $attribs]]
     pack $target_cb -side top -anchor w
     pack $widgets(trans:target) -side top -expand 0 -fill x -padx 4
     pack $ta_frame -anchor center -pady 2
@@ -347,13 +229,13 @@ proc Apol_RBAC::transCreate {t_f} {
                                    -textvariable Apol_RBAC::vals(default:sym) \
                                    -helptext "Type or select a role" -autopost 1]
     trace add variable Apol_RBAC::vals(default:use) write \
-        [list Apol_RBAC::toggleCheckbutton $widgets(trans:default) {}]
+        [list Apol_RBAC::_toggleCheckbutton $widgets(trans:default) {}]
     pack $widgets(trans:default_cb) -side top -anchor w
     pack $widgets(trans:default) -side top -expand 0 -fill x -padx 4
     pack $default -side left -padx 4 -pady 2 -expand 0 -fill y
 }
 
-proc Apol_RBAC::bothCreate {b_f} {
+proc Apol_RBAC::_bothCreate {b_f} {
     variable vals
     variable widgets
 
@@ -374,7 +256,7 @@ proc Apol_RBAC::bothCreate {b_f} {
                        -variable Apol_RBAC::vals(source:which) \
                        -value either]
     trace add variable Apol_RBAC::vals(source:use) write \
-        [list Apol_RBAC::toggleCheckbutton $widgets(both:source) [list $which_source $which_any]]
+        [list Apol_RBAC::_toggleCheckbutton $widgets(both:source) [list $which_source $which_any]]
     pack $which_source $which_any -side top -anchor w
     pack $source_cb -side top -anchor w
     pack $widgets(both:source) -side top -expand 0 -fill x -padx 4
@@ -382,7 +264,7 @@ proc Apol_RBAC::bothCreate {b_f} {
     pack $source -side left -padx 4 -pady 2 -expand 0 -anchor nw
 }
 
-proc Apol_RBAC::toggleCheckbutton {cb w name1 name2 ops} {
+proc Apol_RBAC::_toggleCheckbutton {cb w name1 name2 ops} {
     variable vals
 
     if {$vals($name2)} {
@@ -397,17 +279,17 @@ proc Apol_RBAC::toggleCheckbutton {cb w name1 name2 ops} {
         }
     }
 
-    maybeEnableTargetRole
-    maybeEnableDefaultRole
+    _maybeEnableTargetRole
+    _maybeEnableDefaultRole
 }
 
 # called whenever user selects 'as source'/'any' radio button
-proc Apol_RBAC::toggleRoleBox {name1 name2 ops} {
-    maybeEnableTargetRole
-    maybeEnableDefaultRole
+proc Apol_RBAC::_toggleRoleBox {name1 name2 ops} {
+    _maybeEnableTargetRole
+    _maybeEnableDefaultRole
 }
 
-proc Apol_RBAC::maybeEnableTargetRole {} {
+proc Apol_RBAC::_maybeEnableTargetRole {} {
     variable vals
     variable widgets
     if {$vals(source:use) && $vals(source:which) == "either"} {
@@ -420,7 +302,7 @@ proc Apol_RBAC::maybeEnableTargetRole {} {
     }
 }
 
-proc Apol_RBAC::maybeEnableDefaultRole {} {
+proc Apol_RBAC::_maybeEnableDefaultRole {} {
     variable vals
     variable widgets
     if {$vals(source:use) && $vals(source:which) == "either"} {
@@ -433,7 +315,7 @@ proc Apol_RBAC::maybeEnableDefaultRole {} {
     }
 }
 
-proc Apol_RBAC::toggleTASym {name1 name2 ops} {
+proc Apol_RBAC::_toggleTASym {name1 name2 ops} {
     variable vals
     variable widgets
 
@@ -442,19 +324,167 @@ proc Apol_RBAC::toggleTASym {name1 name2 ops} {
         return
     }
     if {$vals(target_type:types) && $vals(target_type:attribs)} {
-        set items [lsort [concat $Apol_Types::typelist $Apol_Types::attriblist]]
+        set items [lsort [concat [Apol_Types::getTypes] [Apol_Types::getAttributes]]]
     } elseif {$vals(target_type:types)} {
-        set items $Apol_Types::typelist
+        set items [Apol_Types::getTypes]
     } else {
-        set items $Apol_Types::attriblist
+        set items [Apol_Types::getAttributes]
     }
     $widgets(trans:target) configure -values $items
 }
 
 # disallow both types and attribs to be deselected
-proc Apol_RBAC::toggleTAPushed {cb} {
+proc Apol_RBAC::_toggleTAPushed {cb} {
     variable vals
     if {!$vals(target_type:types) && !$vals(target_type:attribs)} {
         $cb select
     }
 }
+
+# callback invoked when the user changes which RBAC rule to search
+proc Apol_RBAC::_ruleChanged {name1 name2 ops} {
+    variable vals
+    variable widgets
+    Apol_Widget::clearSearchResults $widgets(results)
+    $widgets(options_pm) raise $vals(rule_selection)
+}
+
+proc Apol_RBAC::_searchRBACs {} {
+    variable vals
+    variable widgets
+
+    Apol_Widget::clearSearchResults $widgets(results)
+    if {![ApolTop::is_policy_open]} {
+        tk_messageBox -icon error -type ok -title "Error" -message "No current policy file is opened."
+        return
+    }
+
+    set raq {}
+    set rtq {}
+
+    if {$vals(rule_selection) == "allow" || $vals(rule_selection) == "both"} {
+        set raq [new_apol_role_allow_query_t]
+    }
+    if {$vals(rule_selection) == "trans" || $vals(rule_selection) == "both"} {
+        set rtq [new_apol_role_trans_query_t]
+    }
+
+    set source_sym {}
+    set is_any 0
+    if {$vals(source:use)} {
+        if {$vals(source:sym) == {}} {
+            tk_messageBox -icon error -type ok -title "Error" -message "No source role selected."
+            return
+        }
+        if {$vals(source:which) == "either"} {
+            set is_any 1
+        }
+        set source_sym $vals(source:sym)
+    }
+
+    set target_role {}
+    set target_type {}
+    if {$vals(rule_selection) == "allow" && $vals(target_role:use) && \
+            (!$vals(source:use) || $vals(source:which) != "either")} {
+        if {$vals(target_role:sym) == {}} {
+            tk_messageBox -icon error -type ok -title "Error" -message "No target role selected."
+            return
+        }
+        set target_role $vals(target_role:sym)
+    }
+    if {$vals(rule_selection) == "trans" && $vals(target_type:use)} {
+        if {$vals(target_type:sym) == {}} {
+            tk_messageBox -icon error -type ok -title "Error" -message "No target type selected."
+            return
+        }
+        set target_type $vals(target_type:sym)
+    }
+    
+    set default_role {}
+    if {$vals(rule_selection) == "trans" && $vals(default:use) && \
+            (!$vals(source:use) || $vals(source:which) != "either")} {
+        if {$vals(default:sym) == {}} {
+            tk_messageBox -icon error -type ok -title "Error" -message "No default role selected."
+            return
+        }
+        set default_role $vals(default:sym)
+    }
+
+    set role_allows {}
+    if {$raq != {}} {
+        $raq set_source $::ApolTop::policy $source_sym
+        $raq set_source_any $::ApolTop::policy $is_any
+        $raq set_target $::ApolTop::policy $target_role
+        set v [$raq run $::ApolTop::policy]
+        $raq -acquire
+        $raq -delete
+        set role_allows [role_allow_vector_to_list $v]
+        $v -acquire
+        $v -delete
+    }
+
+    set role_trans {}
+    if {$rtq != {}} {
+        $rtq set_source $::ApolTop::policy $source_sym
+        $rtq set_source_any $::ApolTop::policy $is_any
+        $rtq set_target $::ApolTop::policy $target_type 0
+        $rtq set_default $::ApolTop::policy $default_role
+        set v [$rtq run $::ApolTop::policy]
+        $rtq -acquire
+        $rtq -delete
+        set role_trans [role_trans_vector_to_list $v]
+        $v -acquire
+        $v -delete
+    }
+
+    set num_results [expr {[llength $role_allows] + [llength $role_trans]}]
+    if {$num_results == 0} {
+        set text "Search returned no results."
+    } else {
+        set text "$num_results rule"
+        if {$num_results != 1} {
+            append text s
+        }
+        append text " match the search criteria.\n\n"
+    }
+
+    foreach r [lsort -command _role_allow_sort $role_allows] {
+        append text "[_render_role_allow $r]\n"
+    }
+    foreach r [lsort -command _role_trans_sort $role_trans] {
+        append text "[_render_role_trans $r]\n"
+    }
+    Apol_Widget::appendSearchResultText $widgets(results) $text
+}
+
+proc Apol_RBAC::_render_role_allow {qpol_role_allow_datum} {
+    apol_role_allow_render $::ApolTop::policy $qpol_role_allow_datum
+}
+
+proc Apol_RBAC::_render_role_trans {qpol_role_trans_datum} {
+    apol_role_trans_render $::ApolTop::policy $qpol_role_trans_datum
+}
+
+proc Apol_RBAC::_role_allow_sort {a b} {
+    set r1 [[$a get_source_role $::ApolTop::qpolicy] get_name $::ApolTop::qpolicy]
+    set r2 [[$b get_source_role $::ApolTop::qpolicy] get_name $::ApolTop::qpolicy]
+    if {[set z [string compare $r1 $r2]] != 0} {
+        return $z
+    }
+
+    set r1 [[$a get_target_role $::ApolTop::qpolicy] get_name $::ApolTop::qpolicy]
+    set r2 [[$b get_target_role $::ApolTop::qpolicy] get_name $::ApolTop::qpolicy]
+    string compare $r1 $r2
+}
+
+proc Apol_RBAC::_role_trans_sort {a b} {
+    set r1 [[$a get_source_role $::ApolTop::qpolicy] get_name $::ApolTop::qpolicy]
+    set r2 [[$b get_source_role $::ApolTop::qpolicy] get_name $::ApolTop::qpolicy]
+    if {[set z [string compare $r1 $r2]] != 0} {
+        return $z
+    }
+
+    set r1 [[$a get_target_type $::ApolTop::qpolicy] get_name $::ApolTop::qpolicy]
+    set r2 [[$b get_target_type $::ApolTop::qpolicy] get_name $::ApolTop::qpolicy]
+    string compare $r1 $r2
+}
