@@ -31,6 +31,7 @@
 #include <assert.h>
 #include <config.h>
 #include <errno.h>
+#include <limits.h>
 #include <time.h>
 
 /*
@@ -79,7 +80,7 @@ struct apol_infoflow_graph
 
 struct apol_infoflow_node
 {
-	qpol_type_t *type;
+	const qpol_type_t *type;
 	/** one of APOL_INFOFLOW_NODE_SOURCE or APOL_INFOFLOW_NODE_TARGET */
 	int node_type;
 	/** vector of apol_infoflow_edge_t, pointing into the graph */
@@ -129,7 +130,7 @@ struct apol_infoflow_analysis
  */
 struct apol_infoflow_result
 {
-	qpol_type_t *start_type, *end_type;
+	const qpol_type_t *start_type, *end_type;
 	/** vector of apol_infoflow_step_t */
 	apol_vector_t *steps;
 	unsigned int direction;
@@ -146,7 +147,7 @@ struct apol_infoflow_result
  */
 struct apol_infoflow_step
 {
-	qpol_type_t *start_type, *end_type;
+	const qpol_type_t *start_type, *end_type;
 	/** vector of qpol_avrule_t */
 	apol_vector_t *rules;
 	int weight;
@@ -227,7 +228,7 @@ static void apol_infoflow_node_free(void *data)
 
 struct apol_infoflow_node_key
 {
-	qpol_type_t *type;
+	const qpol_type_t *type;
 	int node_type;
 };
 
@@ -254,7 +255,7 @@ static int apol_infoflow_node_compare(const void *a, const void *b __attribute__
 /**
  * Attempt to allocate a new node, add it to the infoflow graph, and
  * return a pointer to it.  If there already exists a node with the
- * same type and object class then reuse that node.
+ * same type then reuse that node.
  *
  * @param p Policy handler, for reporting error.
  * @param g Infoflow to which add the node.
@@ -265,8 +266,8 @@ static int apol_infoflow_node_compare(const void *a, const void *b __attribute__
  * @return Pointer an allocated node within the infoflow graph, or
  * NULL upon error.
  */
-static apol_infoflow_node_t *apol_infoflow_graph_create_node(apol_policy_t * p,
-							     apol_infoflow_graph_t * g, qpol_type_t * type, int node_type)
+static apol_infoflow_node_t *apol_infoflow_graph_create_node(const apol_policy_t * p,
+							     apol_infoflow_graph_t * g, const qpol_type_t * type, int node_type)
 {
 	struct apol_infoflow_node_key key = { type, node_type };
 	apol_infoflow_node_t *node = NULL;
@@ -292,12 +293,14 @@ static apol_infoflow_node_t *apol_infoflow_graph_create_node(apol_policy_t * p,
 /**
  * Attempt to allocate a new node, add it to the infoflow graph, and
  * return a pointer to it.  If there already exists a node with the
- * same type and object class then reuse that node.
+ * same type then reuse that node.
  *
  * @param p Policy handler, for reporting error.
  * @param g Infoflow to which add the node.
  * @param type Type for the new node.  If this is an attribute then it
  * will be expanded into its component types.
+ * @param types If non-NULL, a BST of qpol_type_t pointers.  Only
+ * create and return nodes which are members of this tree.
  * @param node_type Node type, one of APOL_INFOFLOW_NODE_SOURCE or
  * APOL_INFOFLOW_NODE_TARGET.
  *
@@ -305,8 +308,9 @@ static apol_infoflow_node_t *apol_infoflow_graph_create_node(apol_policy_t * p,
  * infoflow graph, or NULL upon error.  The caller is responsible for
  * calling apol_vector_destroy() upon the return value.
  */
-static apol_vector_t *apol_infoflow_graph_create_nodes(apol_policy_t * p,
-						       apol_infoflow_graph_t * g, qpol_type_t * type, int node_type)
+static apol_vector_t *apol_infoflow_graph_create_nodes(const apol_policy_t * p,
+						       apol_infoflow_graph_t * g, const qpol_type_t * type, apol_bst_t * types,
+						       int node_type)
 {
 	unsigned char isattr;
 	apol_vector_t *v = NULL;
@@ -326,6 +330,10 @@ static apol_vector_t *apol_infoflow_graph_create_nodes(apol_policy_t * p,
 		}
 		for (; !qpol_iterator_end(iter); qpol_iterator_next(iter)) {
 			qpol_iterator_get_item(iter, (void **)&t);
+			void *result;
+			if (types != NULL && apol_bst_get_element(types, t, NULL, &result) < 0) {
+				continue;
+			}
 			if ((node = apol_infoflow_graph_create_node(p, g, t, node_type)) == NULL || apol_vector_append(v, node) < 0) {
 				qpol_iterator_destroy(&iter);
 				apol_vector_destroy(&v);
@@ -334,11 +342,14 @@ static apol_vector_t *apol_infoflow_graph_create_nodes(apol_policy_t * p,
 		}
 		qpol_iterator_destroy(&iter);
 	} else {
-		/* for a direct search, do not expand types; the algorithm
-		 * will do that with apol_infoflow_graph_get_nodes_for_type()
-		 * and apol_infoflow_analysis_direct_expand().  for transitive
-		 * searches it is necessary to expand the types so that the
-		 * graph is correctly connected */
+		/* for a direct search, do not expand types; the
+		 * algorithm will do that with
+		 * apol_infoflow_graph_get_nodes_for_type() and
+		 * apol_infoflow_analysis_direct_expand().  for
+		 * transitive searches the \a types BST was checked in
+		 * apol_infoflow_graph_check_types() if \a type is
+		 * just a type.
+		 */
 		if ((v = apol_vector_create_with_capacity(1, NULL)) == NULL) {
 			return NULL;
 		}
@@ -389,7 +400,6 @@ static int apol_infoflow_edge_compare(const void *a, const void *b __attribute__
 	struct apol_infoflow_edge_key *key = (struct apol_infoflow_edge_key *)data;
 	if (key->start_node != NULL && edge->start_node != key->start_node) {
 		return (int)((char *)edge->start_node - (char *)key->start_node);
-		return -1;
 	}
 	if (key->end_node != NULL && edge->end_node != key->end_node) {
 		return (int)((char *)edge->end_node - (char *)key->end_node);
@@ -411,8 +421,8 @@ static int apol_infoflow_edge_compare(const void *a, const void *b __attribute__
  * @return Pointer an allocated node within the infoflow graph, or
  * NULL upon error.
  */
-static apol_infoflow_edge_t *apol_infoflow_graph_create_edge(apol_policy_t * p,
-							     apol_infoflow_graph_t * g,
+static apol_infoflow_edge_t *apol_infoflow_graph_create_edge(const apol_policy_t * p,
+							     apol_infoflow_graph_t * g __attribute__ ((unused)),
 							     apol_infoflow_node_t * start_node,
 							     apol_infoflow_node_t * end_node, int len)
 {
@@ -426,9 +436,10 @@ static apol_infoflow_edge_t *apol_infoflow_graph_create_edge(apol_policy_t * p,
 		}
 		return edge;
 	}
-	if ((edge = calloc(1, sizeof(*edge))) == NULL || (edge->rules = apol_vector_create(NULL)) == NULL) {
+	if ((edge = calloc(1, sizeof(*edge))) == NULL || (edge->rules = apol_vector_create(NULL)) == NULL ||
+	    apol_vector_append(g->edges, edge) < 0) {
+		ERR(p, "%s", strerror(errno));
 		apol_infoflow_edge_free(edge);
-		ERR(p, "%s", strerror(ENOMEM));
 		return NULL;
 	}
 	edge->start_node = start_node;
@@ -436,7 +447,7 @@ static apol_infoflow_edge_t *apol_infoflow_graph_create_edge(apol_policy_t * p,
 	edge->length = len;
 	if (apol_vector_append(start_node->out_edges, edge) < 0 || apol_vector_append(end_node->in_edges, edge) < 0) {
 		/* don't free the edge -- it is owned by the graph */
-		ERR(p, "%s", strerror(ENOMEM));
+		ERR(p, "%s", strerror(errno));
 		return NULL;
 	}
 	return edge;
@@ -446,14 +457,17 @@ static apol_infoflow_edge_t *apol_infoflow_graph_create_edge(apol_policy_t * p,
 
 /**
  * Take an avrule within a policy and possibly add it to the infoflow
- * graph.  The rule must refer to types that are within the types
- * vector.  If the rule is to be added, then add its end nodes as
- * necessary, and an edge connecting those nodes as necessary, and
- * then add the rule to the edge.
+ * graph.  The rule's source and target type sets are expanded.  If
+ * the rule is to be added, then add its end nodes as necessary, and
+ * an edge connecting those nodes as necessary, and then add the rule
+ * to the edge.
  *
  * @param p Policy containing rules.
  * @param g Information flow graph being created.
  * @param rule AV rule to use.
+ * @param types BST of qpol_type_t pointers; while adding avrules to
+ * the graph, only add those whose source and/or target is a member of
+ * \a types, if \a types is non-NULL.
  * @param found_read Non-zero to indicate that this rule performs a
  * read operation.
  * @param read_len Length of the edge to create (proportionally
@@ -465,11 +479,12 @@ static apol_infoflow_edge_t *apol_infoflow_graph_create_edge(apol_policy_t * p,
  *
  * @return 0 on success, < 0 on error.
  */
-static int apol_infoflow_graph_connect_nodes(apol_policy_t * p,
+static int apol_infoflow_graph_connect_nodes(const apol_policy_t * p,
 					     apol_infoflow_graph_t * g,
-					     qpol_avrule_t * rule, int found_read, int read_len, int found_write, int write_len)
+					     const qpol_avrule_t * rule,
+					     apol_bst_t * types, int found_read, int read_len, int found_write, int write_len)
 {
-	qpol_type_t *src_type, *tgt_type;
+	const qpol_type_t *src_type, *tgt_type;
 	apol_vector_t *src_nodes = NULL, *tgt_nodes = NULL;
 	size_t i, j;
 	apol_infoflow_node_t *src_node, *tgt_node;
@@ -480,10 +495,10 @@ static int apol_infoflow_graph_connect_nodes(apol_policy_t * p,
 		goto cleanup;
 	}
 
-	if ((src_nodes = apol_infoflow_graph_create_nodes(p, g, src_type, APOL_INFOFLOW_NODE_SOURCE)) == NULL) {
+	if ((src_nodes = apol_infoflow_graph_create_nodes(p, g, src_type, types, APOL_INFOFLOW_NODE_SOURCE)) == NULL) {
 		goto cleanup;
 	}
-	if ((tgt_nodes = apol_infoflow_graph_create_nodes(p, g, tgt_type, APOL_INFOFLOW_NODE_TARGET)) == NULL) {
+	if ((tgt_nodes = apol_infoflow_graph_create_nodes(p, g, tgt_type, types, APOL_INFOFLOW_NODE_TARGET)) == NULL) {
 		goto cleanup;
 	}
 	for (i = 0; i < apol_vector_get_size(src_nodes); i++) {
@@ -494,7 +509,7 @@ static int apol_infoflow_graph_connect_nodes(apol_policy_t * p,
 				if ((edge = apol_infoflow_graph_create_edge(p, g, tgt_node, src_node, read_len)) == NULL) {
 					goto cleanup;
 				}
-				if (apol_vector_append(edge->rules, rule) < 0) {
+				if (apol_vector_append(edge->rules, (void *)rule) < 0) {
 					ERR(p, "%s", strerror(ENOMEM));
 					goto cleanup;
 				}
@@ -503,7 +518,7 @@ static int apol_infoflow_graph_connect_nodes(apol_policy_t * p,
 				if ((edge = apol_infoflow_graph_create_edge(p, g, src_node, tgt_node, write_len)) == NULL) {
 					goto cleanup;
 				}
-				if (apol_vector_append(edge->rules, rule) < 0) {
+				if (apol_vector_append(edge->rules, (void *)rule) < 0) {
 					ERR(p, "%s", strerror(ENOMEM));
 					goto cleanup;
 				}
@@ -524,17 +539,22 @@ static int apol_infoflow_graph_connect_nodes(apol_policy_t * p,
  * @param p Policy from which to create the infoflow graph.
  * @param g Infoflow graph being created.
  * @param rule AV rule to add.
+ * @param types BST of qpol_type_t pointers; while adding avrules to
+ * the graph, only add those whose source and/or target is a member of
+ * \a types, if \a types is non-NULL.
  * @param max_len Maximum permission length (i.e., inverse of
  * permission weight) to consider when deciding to add this rule or
  * not.
  *
  * @return 0 on success, < 0 on error.
  */
-static int apol_infoflow_graph_create_avrule(apol_policy_t * p, apol_infoflow_graph_t * g, qpol_avrule_t * rule, int max_len)
+static int apol_infoflow_graph_create_avrule(const apol_policy_t * p, apol_infoflow_graph_t * g, const qpol_avrule_t * rule,
+					     apol_bst_t * types, int max_len)
 {
-	qpol_class_t *obj_class;
+	const qpol_class_t *obj_class;
 	qpol_iterator_t *perm_iter = NULL;
-	char *obj_class_name, *perm_name;
+	const char *obj_class_name;
+	char *perm_name;
 	int found_read = 0, found_write = 0, perm_error = 0;
 	int read_len = INT_MAX, write_len = INT_MAX;
 	int retval = -1;
@@ -550,7 +570,7 @@ static int apol_infoflow_graph_create_avrule(apol_policy_t * p, apol_infoflow_gr
 		if (qpol_iterator_get_item(perm_iter, (void **)&perm_name) < 0) {
 			goto cleanup;
 		}
-		if (apol_permmap_get(p, obj_class_name, perm_name, &perm_map, &perm_weight) < 0) {
+		if (apol_policy_get_permmap(p, obj_class_name, perm_name, &perm_map, &perm_weight) < 0) {
 			goto cleanup;
 		}
 		free(perm_name);
@@ -580,7 +600,7 @@ static int apol_infoflow_graph_create_avrule(apol_policy_t * p, apol_infoflow_gr
 
 	/* if we have found any flows then connect them within the graph */
 	if ((found_read || found_write) &&
-	    apol_infoflow_graph_connect_nodes(p, g, rule, found_read, read_len, found_write, write_len) < 0) {
+	    apol_infoflow_graph_connect_nodes(p, g, rule, types, found_read, read_len, found_write, write_len) < 0) {
 		goto cleanup;
 	}
 	if (perm_error) {
@@ -594,22 +614,25 @@ static int apol_infoflow_graph_create_avrule(apol_policy_t * p, apol_infoflow_gr
 }
 
 /**
- * Given a vector of strings representing types, return a vector of
- * qpol_type_t consisting of those types, those types' attributes, and
- * those types' aliases.
+ * Given a vector of strings representing types, return a BST of
+ * qpol_type_t pointers consisting of those types, those types'
+ * attributes, and those types' aliases.
  *
  * @param p Policy within which to look up types,
  * @param v Vector of type strings.
  *
- * @return Vector of qpol_type_t pointers, or NULL on error.
+ * @return BST of qpol_type_t pointers, or NULL on error.  The caller
+ * is responsible for calling apol_bst_destroy() upon the returned
+ * value.
  */
-static apol_vector_t *apol_infoflow_graph_create_required_types(apol_policy_t * p, apol_vector_t * v)
+static apol_bst_t *apol_infoflow_graph_create_required_types(const apol_policy_t * p, const apol_vector_t * v)
 {
-	apol_vector_t *types = NULL, *expanded_types = NULL;
+	apol_bst_t *types = NULL;
+	apol_vector_t *expanded_types = NULL;
 	size_t i;
 	char *s;
 	int retval = -1;
-	if ((types = apol_vector_create(NULL)) == NULL) {
+	if ((types = apol_bst_create(NULL, NULL)) == NULL) {
 		ERR(p, "%s", strerror(errno));
 		goto cleanup;
 	}
@@ -619,18 +642,20 @@ static apol_vector_t *apol_infoflow_graph_create_required_types(apol_policy_t *
 		if (expanded_types == NULL) {
 			goto cleanup;
 		}
-		if (apol_vector_cat(types, expanded_types) < 0) {
-			ERR(p, "%s", strerror(ENOMEM));
-			goto cleanup;
+		for (size_t j = 0; j < apol_vector_get_size(expanded_types); j++) {
+			qpol_type_t *t = (qpol_type_t *) apol_vector_get_element(expanded_types, j);
+			if (apol_bst_insert(types, t, NULL) < 0) {
+				ERR(p, "%s", strerror(errno));
+				goto cleanup;
+			}
 		}
 		apol_vector_destroy(&expanded_types);
 	}
-	apol_vector_sort_uniquify(types, NULL, NULL);
 	retval = 0;
       cleanup:
 	apol_vector_destroy(&expanded_types);
 	if (retval != 0) {
-		apol_vector_destroy(&types);
+		apol_bst_destroy(&types);
 	}
 	return types;
 }
@@ -641,25 +666,24 @@ static apol_vector_t *apol_infoflow_graph_create_required_types(apol_policy_t *
  *
  * @param p Policy to which look up classes and permissions.
  * @param rule AV rule to check.
- * @param types Vector of qpol_type_t, of which both the source and
- * target types must be members.  If NULL or empty then allow all
- * types.
+ * @param types BST of qpol_type_t, of which both the source and
+ * target types must be members.  If NULL allow all types.
  *
  * @return 1 if rule matches, 0 if not, < 0 on error.
  */
-static int apol_infoflow_graph_check_types(apol_policy_t * p, qpol_avrule_t * rule, apol_vector_t * types)
+static int apol_infoflow_graph_check_types(const apol_policy_t * p, const qpol_avrule_t * rule, const apol_bst_t * types)
 {
-	qpol_type_t *source, *target;
-	size_t i;
+	const qpol_type_t *source, *target;
+	void *result;
 	int retval = -1;
-	if (types == NULL || apol_vector_get_size(types) == 0) {
+	if (types == NULL) {
 		retval = 1;
 		goto cleanup;
 	}
 	if (qpol_avrule_get_source_type(p->p, rule, &source) < 0 || qpol_avrule_get_target_type(p->p, rule, &target) < 0) {
 		goto cleanup;
 	}
-	if (apol_vector_get_index(types, source, NULL, NULL, &i) < 0 || apol_vector_get_index(types, target, NULL, NULL, &i) < 0) {
+	if (apol_bst_get_element(types, source, NULL, &result) < 0 || apol_bst_get_element(types, target, NULL, &result) < 0) {
 		retval = 0;
 		goto cleanup;
 	}
@@ -681,10 +705,12 @@ static int apol_infoflow_graph_check_types(apol_policy_t * p, qpol_avrule_t * ru
  *
  * @return 1 if rule matches, 0 if not, < 0 on error.
  */
-static int apol_infoflow_graph_check_class_perms(apol_policy_t * p, qpol_avrule_t * rule, apol_vector_t * class_perms)
+static int apol_infoflow_graph_check_class_perms(const apol_policy_t * p, const qpol_avrule_t * rule,
+						 const apol_vector_t * class_perms)
 {
-	qpol_class_t *obj_class;
-	char *obj_name, *perm;
+	const qpol_class_t *obj_class;
+	const char *obj_name;
+	char *perm;
 	qpol_iterator_t *iter = NULL;
 	apol_obj_perm_t *obj_perm = NULL;
 	apol_vector_t *obj_perm_v = NULL;
@@ -742,9 +768,9 @@ static int apol_infoflow_graph_check_class_perms(apol_policy_t * p, qpol_avrule_
  * @return 0 if the graph was created, < 0 on error.  Upon error *g
  * will be set to NULL.
  */
-static int apol_infoflow_graph_create(apol_policy_t * p, apol_infoflow_analysis_t * ia, apol_infoflow_graph_t ** g)
+static int apol_infoflow_graph_create(const apol_policy_t * p, const apol_infoflow_analysis_t * ia, apol_infoflow_graph_t ** g)
 {
-	apol_vector_t *types = NULL;
+	apol_bst_t *types = NULL;
 	qpol_iterator_t *iter = NULL;
 	int max_len = APOL_PERMMAP_MAX_WEIGHT - ia->min_weight + 1;
 	int compval, retval = -1;
@@ -754,6 +780,8 @@ static int apol_infoflow_graph_create(apol_policy_t * p, apol_infoflow_analysis_
 		ERR(p, "%s", "A permission map must be loaded prior to building the infoflow graph.");
 		goto cleanup;
 	}
+
+	INFO(p, "%s", "Generating information flow graph.");
 	if (ia->mode == APOL_INFOFLOW_MODE_TRANS && ia->intermed != NULL &&
 	    (types = apol_infoflow_graph_create_required_types(p, ia->intermed)) == NULL) {
 		goto cleanup;
@@ -772,6 +800,11 @@ static int apol_infoflow_graph_create(apol_policy_t * p, apol_infoflow_analysis_
 			goto cleanup;
 		}
 	}
+	if (((*g)->edges = apol_vector_create(apol_infoflow_edge_free)) == NULL) {
+		ERR(p, "%s", strerror(errno));
+		goto cleanup;
+	}
+
 	if (qpol_policy_get_avrule_iter(p->p, QPOL_RULE_ALLOW, &iter) < 0) {
 		goto cleanup;
 	}
@@ -793,7 +826,7 @@ static int apol_infoflow_graph_create(apol_policy_t * p, apol_infoflow_analysis_
 		} else if (compval == 0) {
 			continue;
 		}
-		if (apol_infoflow_graph_create_avrule(p, *g, rule, max_len) < 0) {
+		if (apol_infoflow_graph_create_avrule(p, *g, rule, types, max_len) < 0) {
 			goto cleanup;
 		}
 	}
@@ -805,7 +838,7 @@ static int apol_infoflow_graph_create(apol_policy_t * p, apol_infoflow_analysis_
 	apol_bst_destroy(&(*g)->nodes_bst);
 	retval = 0;
       cleanup:
-	apol_vector_destroy(&types);
+	apol_bst_destroy(&types);
 	qpol_iterator_destroy(&iter);
 	if (retval < 0) {
 		apol_infoflow_graph_destroy(g);
@@ -818,6 +851,7 @@ void apol_infoflow_graph_destroy(apol_infoflow_graph_t ** g)
 	if (g != NULL && *g != NULL) {
 		apol_bst_destroy(&(*g)->nodes_bst);
 		apol_vector_destroy(&(*g)->nodes);
+		apol_vector_destroy(&(*g)->edges);
 		apol_vector_destroy(&(*g)->further_start);
 		apol_vector_destroy(&(*g)->further_end);
 		apol_regex_destroy(&(*g)->regex);
@@ -841,7 +875,8 @@ void apol_infoflow_graph_destroy(apol_infoflow_graph_t ** g)
  *
  * @return 0 on success, < 0 on error.
  */
-static int apol_infoflow_graph_get_nodes_for_type(apol_policy_t * p, apol_infoflow_graph_t * g, const char *type, apol_vector_t * v)
+static int apol_infoflow_graph_get_nodes_for_type(const apol_policy_t * p, const apol_infoflow_graph_t * g, const char *type,
+						  apol_vector_t * v)
 {
 	size_t i, j;
 	apol_vector_t *cand_list = NULL;
@@ -875,8 +910,9 @@ static int apol_infoflow_graph_get_nodes_for_type(apol_policy_t * p, apol_infofl
  *
  * @return A usable infoflow result object, or NULL upon error.
  */
-static apol_infoflow_result_t *apol_infoflow_direct_get_result(apol_policy_t * p,
-							       apol_vector_t * v, qpol_type_t * start_type, qpol_type_t * end_type)
+static apol_infoflow_result_t *apol_infoflow_direct_get_result(const apol_policy_t * p,
+							       apol_vector_t * v, const qpol_type_t * start_type,
+							       const qpol_type_t * end_type)
 {
 	size_t i;
 	apol_infoflow_result_t *r;
@@ -908,8 +944,8 @@ static apol_infoflow_result_t *apol_infoflow_direct_get_result(apol_policy_t * p
  *
  * @return 0 on success, < 0 on error.
  */
-static int apol_infoflow_direct_define(apol_policy_t * p,
-				       apol_infoflow_edge_t * edge, unsigned int direction, apol_infoflow_result_t * result)
+static int apol_infoflow_direct_define(const apol_policy_t * p,
+				       const apol_infoflow_edge_t * edge, unsigned int direction, apol_infoflow_result_t * result)
 {
 	apol_infoflow_step_t *step = NULL;
 	if (apol_vector_get_size(result->steps) == 0) {
@@ -930,7 +966,8 @@ static int apol_infoflow_direct_define(apol_policy_t * p,
 		return -1;
 	}
 	result->direction |= direction;
-	if (edge->length < result->length) {
+	//TODO: check that edge->lenght can be safely unsigned
+	if (edge->length < (int)result->length) {
 		result->length = edge->length;
 	}
 	return 0;
@@ -947,9 +984,9 @@ static int apol_infoflow_direct_define(apol_policy_t * p,
  *
  * @return 1 if comparison succeeds, 0 if not, < 0 on error.
  */
-static int apol_infoflow_graph_compare(apol_policy_t * p, apol_infoflow_graph_t * g, qpol_type_t * type)
+static int apol_infoflow_graph_compare(const apol_policy_t * p, apol_infoflow_graph_t * g, const qpol_type_t * type)
 {
-	char *type_name;
+	const char *type_name;
 	qpol_iterator_t *alias_iter = NULL;
 	int compval = 0;
 	if (g->regex == NULL) {
@@ -994,8 +1031,8 @@ static int apol_infoflow_graph_compare(apol_policy_t * p, apol_infoflow_graph_t
  *
  * @return 0 on success, < 0 on error.
  */
-static int apol_infoflow_results_check_both(apol_policy_t * p,
-					    apol_vector_t * working_results, unsigned int direction, apol_vector_t * results)
+static int apol_infoflow_results_check_both(const apol_policy_t * p,
+					    const apol_vector_t * working_results, unsigned int direction, apol_vector_t * results)
 {
 	size_t i;
 	apol_infoflow_result_t *r, *new_r;
@@ -1038,7 +1075,7 @@ static int apol_infoflow_results_check_both(apol_policy_t * p,
  *
  * @return 0 on success, < 0 on error.
  */
-static int apol_infoflow_analysis_direct_expand(apol_policy_t * p,
+static int apol_infoflow_analysis_direct_expand(const apol_policy_t * p,
 						apol_infoflow_graph_t * g,
 						apol_infoflow_node_t * start_node,
 						apol_infoflow_edge_t * edge, unsigned int flow_dir, apol_vector_t * results)
@@ -1046,7 +1083,7 @@ static int apol_infoflow_analysis_direct_expand(apol_policy_t * p,
 	apol_infoflow_node_t *end_node;
 	unsigned char isattr;
 	qpol_iterator_t *iter = NULL;
-	qpol_type_t *type;
+	const qpol_type_t *type;
 	apol_infoflow_result_t *r;
 	int retval = -1, compval;
 
@@ -1108,7 +1145,7 @@ static int apol_infoflow_analysis_direct_expand(apol_policy_t * p,
  *
  * @return 0 on success, < 0 on error.
  */
-static int apol_infoflow_analysis_direct(apol_policy_t * p,
+static int apol_infoflow_analysis_direct(const apol_policy_t * p,
 					 apol_infoflow_graph_t * g, const char *start_type, apol_vector_t * results)
 {
 	apol_vector_t *nodes = NULL;
@@ -1174,7 +1211,7 @@ static int apol_infoflow_analysis_direct(apol_policy_t * p,
  *
  * @return 0 on success, < 0 on error.
  */
-static int apol_infoflow_graph_trans_init(apol_policy_t * p,
+static int apol_infoflow_graph_trans_init(const apol_policy_t * p,
 					  apol_infoflow_graph_t * g, apol_infoflow_node_t * start, apol_queue_t * q)
 {
 	size_t i;
@@ -1209,7 +1246,7 @@ static int apol_infoflow_graph_trans_init(apol_policy_t * p,
  *
  * @return 0 on success, < 0 on error.
  */
-static int apol_infoflow_graph_trans_further_init(apol_policy_t * p,
+static int apol_infoflow_graph_trans_further_init(const apol_policy_t * p,
 						  apol_infoflow_graph_t * g, apol_infoflow_node_t * start, apol_queue_t * q)
 {
 	size_t i;
@@ -1249,7 +1286,7 @@ static int apol_infoflow_graph_trans_further_init(apol_policy_t * p,
  *
  * @return 0 on success, < 0 on error.
  */
-static int apol_infoflow_trans_path(apol_policy_t * p,
+static int apol_infoflow_trans_path(const apol_policy_t * p,
 				    apol_infoflow_graph_t * g,
 				    apol_infoflow_node_t * start_node, apol_infoflow_node_t * end_node, apol_vector_t ** path)
 {
@@ -1293,7 +1330,7 @@ static int apol_infoflow_trans_path(apol_policy_t * p,
  *
  * @return Edge connecting node to next_node, or NULL on error.
  */
-static apol_infoflow_edge_t *apol_infoflow_trans_find_edge(apol_policy_t * p,
+static apol_infoflow_edge_t *apol_infoflow_trans_find_edge(const apol_policy_t * p,
 							   apol_infoflow_graph_t * g,
 							   apol_infoflow_node_t * node, apol_infoflow_node_t * next_node)
 {
@@ -1340,9 +1377,9 @@ static apol_infoflow_edge_t *apol_infoflow_trans_find_edge(apol_policy_t * p,
  *
  * @return 0 on success, < 0 on error.
  */
-static int apol_infoflow_trans_define(apol_policy_t * p,
+static int apol_infoflow_trans_define(const apol_policy_t * p,
 				      apol_infoflow_graph_t * g,
-				      apol_vector_t * path, qpol_type_t * end_type, apol_infoflow_result_t ** result)
+				      apol_vector_t * path, const qpol_type_t * end_type, apol_infoflow_result_t ** result)
 {
 	apol_infoflow_step_t *step = NULL;
 	size_t path_len = apol_vector_get_size(path), i;
@@ -1430,9 +1467,9 @@ static int apol_infoflow_trans_step_comp(const void *a, const void *b, void *dat
  *
  * @return 0 on success, < 0 on error.
  */
-static int apol_infoflow_trans_append(apol_policy_t * p,
+static int apol_infoflow_trans_append(const apol_policy_t * p,
 				      apol_infoflow_graph_t * g,
-				      apol_vector_t * path, qpol_type_t * end_type, apol_vector_t * results)
+				      apol_vector_t * path, const qpol_type_t * end_type, apol_vector_t * results)
 {
 	apol_infoflow_result_t *new_r = NULL, *r;
 	size_t i, j;
@@ -1474,22 +1511,22 @@ static int apol_infoflow_trans_append(apol_policy_t * p,
 
 /**
  * Given a start and end node, add a trans infoflow results to a
- * vector.  If the end node is an attribute, first expand the
- * attribute to its component types.  If a regular expression is
- * compiled into the infoflow graph, apply that regex match against
- * candidate end node types prior to creating result nodes.
+ * vector.  If a regular expression is compiled into the infoflow
+ * graph, apply that regex match against candidate end node types
+ * prior to creating result nodes.
  *
  * @param p Policy to analyze.
  * @param g Information flow graph to analyze.
  * @param start_node Starting node.
  * @param end_node Ending node.
- * @param results Non-NULL vector to which append infoflow results.
+ * @param results Non-NULL vector to which append infoflow result.
  * The caller is responsible for calling apol_infoflow_results_free()
  * upon each element afterwards.
  *
- * @return 0 on success, < 0 on error.
+ * @return 0 on success (including no result actually added), or < 0
+ * on error.
  */
-static int apol_infoflow_analysis_trans_expand(apol_policy_t * p,
+static int apol_infoflow_analysis_trans_expand(const apol_policy_t * p,
 					       apol_infoflow_graph_t * g,
 					       apol_infoflow_node_t * start_node,
 					       apol_infoflow_node_t * end_node, apol_vector_t * results)
@@ -1553,7 +1590,7 @@ static int apol_infoflow_analysis_trans_expand(apol_policy_t * p,
  *
  * @return 0 on success, < 0 on error.
  */
-static int apol_infoflow_analysis_trans_shortest_path(apol_policy_t * p,
+static int apol_infoflow_analysis_trans_shortest_path(const apol_policy_t * p,
 						      apol_infoflow_graph_t * g,
 						      apol_infoflow_node_t * start, apol_vector_t * results)
 {
@@ -1647,7 +1684,7 @@ static int apol_infoflow_analysis_trans_shortest_path(apol_policy_t * p,
  *
  * @return 0 on success, < 0 on error.
  */
-static int apol_infoflow_analysis_trans(apol_policy_t * p,
+static int apol_infoflow_analysis_trans(const apol_policy_t * p,
 					apol_infoflow_graph_t * g, const char *start_type, apol_vector_t * results)
 {
 	apol_vector_t *start_nodes = NULL;
@@ -1691,7 +1728,7 @@ static int apol_infoflow_analysis_trans(apol_policy_t * p,
  * upon error.  The caller must call apol_vector_destroy() upon the
  * returned value.
  */
-static apol_vector_t *apol_infoflow_trans_further_shuffle(apol_policy_t * p, apol_infoflow_graph_t * g, apol_vector_t * v)
+static apol_vector_t *apol_infoflow_trans_further_shuffle(const apol_policy_t * p, apol_infoflow_graph_t * g, apol_vector_t * v)
 {
 	size_t i, j, size;
 	void **deck = NULL, *tmp;
@@ -1734,7 +1771,7 @@ static apol_vector_t *apol_infoflow_trans_further_shuffle(apol_policy_t * p, apo
 	return new_v;
 }
 
-static int apol_infoflow_analysis_trans_further(apol_policy_t * p,
+static int apol_infoflow_analysis_trans_further(const apol_policy_t * p,
 						apol_infoflow_graph_t * g, apol_infoflow_node_t * start, apol_vector_t * results)
 {
 	apol_vector_t *edge_list = NULL;
@@ -1796,7 +1833,8 @@ static int apol_infoflow_analysis_trans_further(apol_policy_t * p,
 
 /******************** infoflow analysis object routines ********************/
 
-int apol_infoflow_analysis_do(apol_policy_t * p, apol_infoflow_analysis_t * ia, apol_vector_t ** v, apol_infoflow_graph_t ** g)
+int apol_infoflow_analysis_do(const apol_policy_t * p, const apol_infoflow_analysis_t * ia, apol_vector_t ** v,
+			      apol_infoflow_graph_t ** g)
 {
 	int retval = -1;
 	if (v != NULL) {
@@ -1812,6 +1850,7 @@ int apol_infoflow_analysis_do(apol_policy_t * p, apol_infoflow_analysis_t * ia,
 	if (apol_infoflow_graph_create(p, ia, g) < 0) {
 		goto cleanup;
 	}
+	INFO(p, "%s", "Searching information flow graph.");
 	retval = apol_infoflow_analysis_do_more(p, *g, ia->type, v);
       cleanup:
 	if (retval != 0) {
@@ -1820,9 +1859,9 @@ int apol_infoflow_analysis_do(apol_policy_t * p, apol_infoflow_analysis_t * ia,
 	return retval;
 }
 
-int apol_infoflow_analysis_do_more(apol_policy_t * p, apol_infoflow_graph_t * g, const char *type, apol_vector_t ** v)
+int apol_infoflow_analysis_do_more(const apol_policy_t * p, apol_infoflow_graph_t * g, const char *type, apol_vector_t ** v)
 {
-	qpol_type_t *start_type;
+	const qpol_type_t *start_type;
 	int retval = -1;
 	if (v != NULL) {
 		*v = NULL;
@@ -1855,10 +1894,10 @@ int apol_infoflow_analysis_do_more(apol_policy_t * p, apol_infoflow_graph_t * g,
 	return retval;
 }
 
-int apol_infoflow_analysis_trans_further_prepare(apol_policy_t * p,
+int apol_infoflow_analysis_trans_further_prepare(const apol_policy_t * p,
 						 apol_infoflow_graph_t * g, const char *start_type, const char *end_type)
 {
-	qpol_type_t *stype, *etype;
+	const qpol_type_t *stype, *etype;
 	int retval = -1;
 
 	apol_infoflow_srand(g);
@@ -1885,7 +1924,7 @@ int apol_infoflow_analysis_trans_further_prepare(apol_policy_t * p,
 	return retval;
 }
 
-int apol_infoflow_analysis_trans_further_next(apol_policy_t * p, apol_infoflow_graph_t * g, apol_vector_t ** v)
+int apol_infoflow_analysis_trans_further_next(const apol_policy_t * p, apol_infoflow_graph_t * g, apol_vector_t ** v)
 {
 	apol_infoflow_node_t *start_node;
 	int retval = -1;
@@ -1931,41 +1970,45 @@ void apol_infoflow_analysis_destroy(apol_infoflow_analysis_t ** ia)
 	}
 }
 
-int apol_infoflow_analysis_set_mode(apol_policy_t * p, apol_infoflow_analysis_t * ia, unsigned int mode)
+int apol_infoflow_analysis_set_mode(const apol_policy_t * p, apol_infoflow_analysis_t * ia, unsigned int mode)
 {
 	switch (mode) {
 	case APOL_INFOFLOW_MODE_DIRECT:
-	case APOL_INFOFLOW_MODE_TRANS:{
-			ia->mode = mode;
-			break;
-		}
-	default:{
-			ERR(p, "%s", strerror(EINVAL));
-			return -1;
-		}
+	case APOL_INFOFLOW_MODE_TRANS:
+	{
+		ia->mode = mode;
+		break;
+	}
+	default:
+	{
+		ERR(p, "%s", strerror(EINVAL));
+		return -1;
+	}
 	}
 	return 0;
 }
 
-int apol_infoflow_analysis_set_dir(apol_policy_t * p, apol_infoflow_analysis_t * ia, unsigned int dir)
+int apol_infoflow_analysis_set_dir(const apol_policy_t * p, apol_infoflow_analysis_t * ia, unsigned int dir)
 {
 	switch (dir) {
 	case APOL_INFOFLOW_IN:
 	case APOL_INFOFLOW_OUT:
 	case APOL_INFOFLOW_BOTH:
-	case APOL_INFOFLOW_EITHER:{
-			ia->direction = dir;
-			break;
-		}
-	default:{
-			ERR(p, "%s", strerror(EINVAL));
-			return -1;
-		}
+	case APOL_INFOFLOW_EITHER:
+	{
+		ia->direction = dir;
+		break;
+	}
+	default:
+	{
+		ERR(p, "%s", strerror(EINVAL));
+		return -1;
+	}
 	}
 	return 0;
 }
 
-int apol_infoflow_analysis_set_type(apol_policy_t * p, apol_infoflow_analysis_t * ia, const char *name)
+int apol_infoflow_analysis_set_type(const apol_policy_t * p, apol_infoflow_analysis_t * ia, const char *name)
 {
 	if (name == NULL) {
 		ERR(p, "%s", strerror(EINVAL));
@@ -1982,7 +2025,7 @@ static int compare_class_perm_by_class_name(const void *in_op, const void *class
 	return strcmp(apol_obj_perm_get_obj_name(op), name);
 }
 
-int apol_infoflow_analysis_append_intermediate(apol_policy_t * policy, apol_infoflow_analysis_t * ia, const char *type)
+int apol_infoflow_analysis_append_intermediate(const apol_policy_t * policy, apol_infoflow_analysis_t * ia, const char *type)
 {
 	char *tmp = NULL;
 	if (type == NULL) {
@@ -2001,7 +2044,7 @@ int apol_infoflow_analysis_append_intermediate(apol_policy_t * policy, apol_info
 	return 0;
 }
 
-int apol_infoflow_analysis_append_class_perm(apol_policy_t * p,
+int apol_infoflow_analysis_append_class_perm(const apol_policy_t * p,
 					     apol_infoflow_analysis_t * ia, const char *class_name, const char *perm_name)
 {
 	apol_obj_perm_t *op = NULL;
@@ -2051,7 +2094,8 @@ int apol_infoflow_analysis_append_class_perm(apol_policy_t * p,
 	return 0;
 }
 
-int apol_infoflow_analysis_set_min_weight(apol_policy_t * p, apol_infoflow_analysis_t * ia, int min_weight)
+int apol_infoflow_analysis_set_min_weight(const apol_policy_t * p
+					  __attribute__ ((unused)), apol_infoflow_analysis_t * ia, int min_weight)
 {
 	if (min_weight <= 0) {
 		ia->min_weight = 0;
@@ -2063,61 +2107,98 @@ int apol_infoflow_analysis_set_min_weight(apol_policy_t * p, apol_infoflow_analy
 	return 0;
 }
 
-int apol_infoflow_analysis_set_result_regex(apol_policy_t * p, apol_infoflow_analysis_t * ia, const char *result)
+int apol_infoflow_analysis_set_result_regex(const apol_policy_t * p, apol_infoflow_analysis_t * ia, const char *result)
 {
 	return apol_query_set(p, &ia->result, NULL, result);
 }
 
 /*************** functions to access infoflow results ***************/
 
-unsigned int apol_infoflow_result_get_dir(apol_infoflow_result_t * result)
+unsigned int apol_infoflow_result_get_dir(const apol_infoflow_result_t * result)
 {
+	if (!result) {
+		errno = EINVAL;
+		return 0;
+	}
 	return result->direction;
 }
 
-qpol_type_t *apol_infoflow_result_get_start_type(apol_infoflow_result_t * result)
+const qpol_type_t *apol_infoflow_result_get_start_type(const apol_infoflow_result_t * result)
 {
+	if (!result) {
+		errno = EINVAL;
+		return NULL;
+	}
 	return result->start_type;
 }
 
-qpol_type_t *apol_infoflow_result_get_end_type(apol_infoflow_result_t * result)
+const qpol_type_t *apol_infoflow_result_get_end_type(const apol_infoflow_result_t * result)
 {
+	if (!result) {
+		errno = EINVAL;
+		return NULL;
+	}
 	return result->end_type;
 }
 
-unsigned int apol_infoflow_result_get_length(apol_infoflow_result_t * result)
+unsigned int apol_infoflow_result_get_length(const apol_infoflow_result_t * result)
 {
+	if (!result) {
+		errno = EINVAL;
+		return 0;
+	}
+	assert(result->length != 0);
 	return result->length;
 }
 
-apol_vector_t *apol_infoflow_result_get_steps(apol_infoflow_result_t * result)
+const apol_vector_t *apol_infoflow_result_get_steps(const apol_infoflow_result_t * result)
 {
+	if (!result) {
+		errno = EINVAL;
+		return NULL;
+	}
 	return result->steps;
 }
 
-qpol_type_t *apol_infoflow_step_get_start_type(apol_infoflow_step_t * step)
+const qpol_type_t *apol_infoflow_step_get_start_type(const apol_infoflow_step_t * step)
 {
+	if (!step) {
+		errno = EINVAL;
+		return NULL;
+	}
 	return step->start_type;
 }
 
-qpol_type_t *apol_infoflow_step_get_end_type(apol_infoflow_step_t * step)
+const qpol_type_t *apol_infoflow_step_get_end_type(const apol_infoflow_step_t * step)
 {
+	if (!step) {
+		errno = EINVAL;
+		return NULL;
+	}
 	return step->end_type;
 }
 
-int apol_infoflow_step_get_weight(apol_infoflow_step_t * step)
+int apol_infoflow_step_get_weight(const apol_infoflow_step_t * step)
 {
+	if (!step) {
+		errno = EINVAL;
+		return -1;
+	}
 	return step->weight;
 }
 
-apol_vector_t *apol_infoflow_step_get_rules(apol_infoflow_step_t * step)
+const apol_vector_t *apol_infoflow_step_get_rules(const apol_infoflow_step_t * step)
 {
+	if (!step) {
+		errno = EINVAL;
+		return NULL;
+	}
 	return step->rules;
 }
 
 /******************** protected functions ********************/
 
-apol_infoflow_result_t *infoflow_result_create_from_infoflow_result(apol_infoflow_result_t * result)
+apol_infoflow_result_t *infoflow_result_create_from_infoflow_result(const apol_infoflow_result_t * result)
 {
 	apol_infoflow_result_t *new_r = NULL;
 	apol_infoflow_step_t *step, *new_step;
@@ -2125,8 +2206,8 @@ apol_infoflow_result_t *infoflow_result_create_from_infoflow_result(apol_infoflo
 	int retval = -1;
 
 	if ((new_r = calloc(1, sizeof(*new_r))) == NULL ||
-	    (new_r->steps =
-	     apol_vector_create_with_capacity(apol_vector_get_size(result->steps), apol_infoflow_step_free)) == NULL) {
+	    (new_r->steps = apol_vector_create_with_capacity(apol_vector_get_size(result->steps), apol_infoflow_step_free)) == NULL)
+	{
 		goto cleanup;
 	}
 	new_r->start_type = result->start_type;
