@@ -67,7 +67,9 @@ struct type_map
  */
 struct poldiff_type_remap_entry
 {
+	/** vector of names of qpol_type_t in original qpolicy */
 	apol_vector_t *orig_types;
+	/** vector of names of qpol_type_t in the modified qpolicy */
 	apol_vector_t *mod_types;
 	int inferred;
 	int enabled;
@@ -101,9 +103,9 @@ static poldiff_type_remap_entry_t *poldiff_type_remap_entry_create(poldiff_t * d
 {
 	poldiff_type_remap_entry_t *e = NULL;
 	if ((e = calloc(1, sizeof(*e))) == NULL ||
-	    (e->orig_types = apol_vector_create_with_capacity(1, NULL)) == NULL ||
-	    (e->mod_types = apol_vector_create_with_capacity(1, NULL)) == NULL
-	    || apol_vector_append(diff->type_map->remap, e) < 0) {
+	    (e->orig_types = apol_vector_create_with_capacity(1, free)) == NULL ||
+	    (e->mod_types = apol_vector_create_with_capacity(1, free)) == NULL || apol_vector_append(diff->type_map->remap, e) < 0)
+	{
 		type_remap_entry_free(e);
 		return NULL;
 	}
@@ -117,7 +119,7 @@ int poldiff_type_remap_create(poldiff_t * diff, const apol_vector_t * orig_names
 	poldiff_type_remap_entry_t *entry = NULL;
 	size_t i;
 	char *name;
-	qpol_type_t *type;
+	const qpol_type_t *type;
 	unsigned char isalias, isattr;
 	int retval = -1, error = 0;
 	if (diff == NULL || orig_names == NULL || mod_names == NULL) {
@@ -132,8 +134,8 @@ int poldiff_type_remap_create(poldiff_t * diff, const apol_vector_t * orig_names
 		goto cleanup;
 	}
 	if ((entry = calloc(1, sizeof(*entry))) == NULL ||
-	    (entry->orig_types = apol_vector_create_with_capacity(1, NULL)) == NULL ||
-	    (entry->mod_types = apol_vector_create_with_capacity(1, NULL)) == NULL) {
+	    (entry->orig_types = apol_vector_create_with_capacity(1, free)) == NULL ||
+	    (entry->mod_types = apol_vector_create_with_capacity(1, free)) == NULL) {
 		error = ENOMEM;
 		ERR(diff, "%s", strerror(error));
 		goto cleanup;
@@ -151,13 +153,14 @@ int poldiff_type_remap_create(poldiff_t * diff, const apol_vector_t * orig_names
 			ERR(diff, "%s is not a primary type.", name);
 			goto cleanup;
 		}
-		if (apol_vector_append(entry->orig_types, type) < 0) {
+		if ((name = strdup(name)) == NULL || apol_vector_append(entry->orig_types, (void *)name) < 0) {
 			error = ENOMEM;
+			free(name);
 			ERR(diff, "%s", strerror(error));
 			goto cleanup;
 		}
 	}
-	apol_vector_sort_uniquify(entry->orig_types, NULL, NULL);
+	apol_vector_sort_uniquify(entry->orig_types, apol_str_strcmp, NULL);
 	for (i = 0; i < apol_vector_get_size(mod_names); i++) {
 		name = (char *)apol_vector_get_element(mod_names, i);
 		if (qpol_policy_get_type_by_name(diff->mod_qpol, name, &type) < 0 ||
@@ -171,13 +174,14 @@ int poldiff_type_remap_create(poldiff_t * diff, const apol_vector_t * orig_names
 			ERR(diff, "%s is not a primary type.", name);
 			goto cleanup;
 		}
-		if (apol_vector_append(entry->mod_types, type) < 0) {
+		if ((name = strdup(name)) == NULL || apol_vector_append(entry->mod_types, (void *)name) < 0) {
 			error = ENOMEM;
+			free(name);
 			ERR(diff, "%s", strerror(error));
 			goto cleanup;
 		}
 	}
-	apol_vector_sort_uniquify(entry->mod_types, NULL, NULL);
+	apol_vector_sort_uniquify(entry->mod_types, apol_str_strcmp, NULL);
 	entry->enabled = 1;
 	if (apol_vector_append(diff->type_map->remap, entry) < 0) {
 		error = ENOMEM;
@@ -194,7 +198,7 @@ int poldiff_type_remap_create(poldiff_t * diff, const apol_vector_t * orig_names
 	return retval;
 }
 
-apol_vector_t *poldiff_type_remap_get_entries(poldiff_t * diff)
+apol_vector_t *poldiff_type_remap_get_entries(const poldiff_t * diff)
 {
 	if (diff == NULL || diff->type_map == NULL) {
 		ERR(diff, "%s", strerror(EINVAL));
@@ -221,81 +225,27 @@ void poldiff_type_remap_entry_remove(poldiff_t * diff, poldiff_type_remap_entry_
 	diff->remapped = 1;
 }
 
-apol_vector_t *poldiff_type_remap_entry_get_original_types(poldiff_t * diff, poldiff_type_remap_entry_t * entry)
+apol_vector_t *poldiff_type_remap_entry_get_original_types(const poldiff_t * diff, const poldiff_type_remap_entry_t * entry)
 {
-	apol_vector_t *v = NULL;
-	int error;
-	size_t i;
 	if (diff == NULL || entry == NULL) {
 		ERR(diff, "%s", strerror(EINVAL));
 		errno = EINVAL;
 		return NULL;
 	}
-	if ((v = apol_vector_create_with_capacity(apol_vector_get_size(entry->orig_types), NULL)) == NULL) {
-		error = errno;
-		ERR(diff, "%s", strerror(error));
-		errno = error;
-		return NULL;
-	}
-	for (i = 0; i < apol_vector_get_size(entry->orig_types); i++) {
-		qpol_type_t *t = (qpol_type_t *) apol_vector_get_element(entry->orig_types, i);
-		char *name;
-		if (qpol_type_get_name(diff->orig_qpol, t, &name) < 0) {
-			error = errno;
-			apol_vector_destroy(&v);
-			errno = error;
-			return NULL;
-		}
-		if (apol_vector_append(v, name) < 0) {
-			error = errno;
-			ERR(diff, "%s", strerror(error));
-			apol_vector_destroy(&v);
-			errno = error;
-			return NULL;
-		}
-	}
-	apol_vector_sort(v, apol_str_strcmp, NULL);
-	return v;
+	return apol_vector_create_from_vector(entry->orig_types, NULL, NULL, NULL);
 }
 
-apol_vector_t *poldiff_type_remap_entry_get_modified_types(poldiff_t * diff, poldiff_type_remap_entry_t * entry)
+apol_vector_t *poldiff_type_remap_entry_get_modified_types(const poldiff_t * diff, const poldiff_type_remap_entry_t * entry)
 {
-	apol_vector_t *v = NULL;
-	int error;
-	size_t i;
 	if (diff == NULL || entry == NULL) {
 		ERR(diff, "%s", strerror(EINVAL));
 		errno = EINVAL;
 		return NULL;
 	}
-	if ((v = apol_vector_create_with_capacity(apol_vector_get_size(entry->mod_types), NULL)) == NULL) {
-		error = errno;
-		ERR(diff, "%s", strerror(error));
-		errno = error;
-		return NULL;
-	}
-	for (i = 0; i < apol_vector_get_size(entry->mod_types); i++) {
-		qpol_type_t *t = (qpol_type_t *) apol_vector_get_element(entry->mod_types, i);
-		char *name;
-		if (qpol_type_get_name(diff->mod_qpol, t, &name) < 0) {
-			error = errno;
-			apol_vector_destroy(&v);
-			errno = error;
-			return NULL;
-		}
-		if (apol_vector_append(v, name) < 0) {
-			error = errno;
-			ERR(diff, "%s", strerror(error));
-			apol_vector_destroy(&v);
-			errno = error;
-			return NULL;
-		}
-	}
-	apol_vector_sort(v, apol_str_strcmp, NULL);
-	return v;
+	return apol_vector_create_from_vector(entry->mod_types, NULL, NULL, NULL);
 }
 
-int poldiff_type_remap_entry_get_is_inferred(poldiff_type_remap_entry_t * entry)
+int poldiff_type_remap_entry_get_is_inferred(const poldiff_type_remap_entry_t * entry)
 {
 	if (entry == NULL) {
 		errno = EINVAL;
@@ -304,7 +254,7 @@ int poldiff_type_remap_entry_get_is_inferred(poldiff_type_remap_entry_t * entry)
 	return entry->inferred;
 }
 
-int poldiff_type_remap_entry_get_is_enabled(poldiff_type_remap_entry_t * entry)
+int poldiff_type_remap_entry_get_is_enabled(const poldiff_type_remap_entry_t * entry)
 {
 	if (entry == NULL) {
 		errno = EINVAL;
@@ -361,11 +311,11 @@ static void type_map_dump(poldiff_t * diff)
 #ifdef SETOOLS_DEBUG
 	size_t i, j;
 	apol_vector_t *v;
-	qpol_type_t *t;
-	char *name;
+	const qpol_type_t *t;
+	const char *name;
 	printf("# type map debug dump (qpol_type_t -> pseudo-type):\norig:\n");
 	for (i = 0; i < diff->type_map->num_orig_types; i++) {
-		printf("%3d:%5d", i, diff->type_map->orig_to_pseudo[i]);
+		printf("%3zd:%5d", i, diff->type_map->orig_to_pseudo[i]);
 		if ((i + 1) % 5 == 0) {
 			printf("\n");
 		} else {
@@ -374,7 +324,7 @@ static void type_map_dump(poldiff_t * diff)
 	}
 	for (i = 0; i < apol_vector_get_size(diff->type_map->pseudo_to_orig); i++) {
 		v = apol_vector_get_element(diff->type_map->pseudo_to_orig, i);
-		printf("\n%3d->", i);
+		printf("\n%3zd->", i);
 		for (j = 0; j < apol_vector_get_size(v); j++) {
 			t = apol_vector_get_element(v, j);
 			qpol_type_get_name(diff->orig_qpol, t, &name);
@@ -383,7 +333,7 @@ static void type_map_dump(poldiff_t * diff)
 	}
 	printf("\nmod:\n");
 	for (i = 0; i < diff->type_map->num_mod_types; i++) {
-		printf("%3d:%5d", i, diff->type_map->mod_to_pseudo[i]);
+		printf("%3zd:%5d", i, diff->type_map->mod_to_pseudo[i]);
 		if ((i + 1) % 5 == 0) {
 			printf("\n");
 		} else {
@@ -392,7 +342,7 @@ static void type_map_dump(poldiff_t * diff)
 	}
 	for (i = 0; i < apol_vector_get_size(diff->type_map->pseudo_to_mod); i++) {
 		v = apol_vector_get_element(diff->type_map->pseudo_to_mod, i);
-		printf("\n%3d->", i);
+		printf("\n%3zd->", i);
 		for (j = 0; j < apol_vector_get_size(v); j++) {
 			t = apol_vector_get_element(v, j);
 			qpol_type_get_name(diff->mod_qpol, t, &name);
@@ -420,15 +370,10 @@ int type_map_build(poldiff_t * diff)
 	apol_vector_t *ov = NULL, *mv = NULL;
 	int retval = -1, error = 0;
 	size_t i, j;
-	qpol_type_t *t;
+	const qpol_type_t *t;
 	uint32_t val, max_val, next_val;
 	apol_vector_t *reverse_v = NULL;
 
-	if (diff == NULL || diff->type_map == NULL) {
-		ERR(diff, "%s", strerror(EINVAL));
-		errno = EINVAL;
-		return -1;
-	}
 	map = diff->type_map;
 	free(map->orig_to_pseudo);
 	map->orig_to_pseudo = NULL;
@@ -492,7 +437,7 @@ int type_map_build(poldiff_t * diff)
 	next_val = 1;
 	for (i = 0; i < apol_vector_get_size(map->remap); i++) {
 		poldiff_type_remap_entry_t *e;
-		char *name;
+		const char *name;
 		e = (poldiff_type_remap_entry_t *) apol_vector_get_element(map->remap, i);
 		if (!e->enabled) {
 			continue;
@@ -504,8 +449,9 @@ int type_map_build(poldiff_t * diff)
 			goto cleanup;
 		}
 		for (j = 0; j < apol_vector_get_size(e->orig_types); j++) {
-			t = (qpol_type_t *) apol_vector_get_element(e->orig_types, j);
-			if (qpol_type_get_value(diff->orig_qpol, t, &val) < 0 || qpol_type_get_name(diff->orig_qpol, t, &name) < 0) {
+			name = (const char *)apol_vector_get_element(e->orig_types, j);
+			if (qpol_policy_get_type_by_name(diff->orig_qpol, name, &t) < 0 ||
+			    qpol_type_get_value(diff->orig_qpol, t, &val) < 0) {
 				error = errno;
 				goto cleanup;
 			}
@@ -515,7 +461,7 @@ int type_map_build(poldiff_t * diff)
 				goto cleanup;
 			}
 			map->orig_to_pseudo[val - 1] = next_val;
-			if (apol_vector_append(reverse_v, t) < 0) {
+			if (apol_vector_append(reverse_v, (void *)t) < 0) {
 				error = errno;
 				ERR(diff, "%s", strerror(error));
 				goto cleanup;
@@ -534,8 +480,9 @@ int type_map_build(poldiff_t * diff)
 			goto cleanup;
 		}
 		for (j = 0; j < apol_vector_get_size(e->mod_types); j++) {
-			t = (qpol_type_t *) apol_vector_get_element(e->mod_types, j);
-			if (qpol_type_get_value(diff->mod_qpol, t, &val) < 0 || qpol_type_get_name(diff->mod_qpol, t, &name) < 0) {
+			name = (const char *)apol_vector_get_element(e->mod_types, j);
+			if (qpol_policy_get_type_by_name(diff->mod_qpol, name, &t) < 0 ||
+			    qpol_type_get_value(diff->mod_qpol, t, &val) < 0) {
 				error = errno;
 				goto cleanup;
 			}
@@ -545,7 +492,7 @@ int type_map_build(poldiff_t * diff)
 				goto cleanup;
 			}
 			map->mod_to_pseudo[val - 1] = next_val;
-			if (apol_vector_append(reverse_v, t) < 0) {
+			if (apol_vector_append(reverse_v, (void *)t) < 0) {
 				error = errno;
 				ERR(diff, "%s", strerror(error));
 				goto cleanup;
@@ -572,7 +519,8 @@ int type_map_build(poldiff_t * diff)
 		if (map->orig_to_pseudo[val - 1] == 0) {
 			map->orig_to_pseudo[val - 1] = next_val;
 			if ((reverse_v = apol_vector_create_with_capacity(1, NULL)) == NULL ||
-			    apol_vector_append(reverse_v, t) < 0 || apol_vector_append(map->pseudo_to_orig, reverse_v) < 0) {
+			    apol_vector_append(reverse_v, (void *)t) < 0 ||
+			    apol_vector_append(map->pseudo_to_orig, reverse_v) < 0) {
 				error = errno;
 				ERR(diff, "%s", strerror(error));
 				goto cleanup;
@@ -597,7 +545,7 @@ int type_map_build(poldiff_t * diff)
 		if (map->mod_to_pseudo[val - 1] == 0) {
 			map->mod_to_pseudo[val - 1] = next_val;
 			if ((reverse_v = apol_vector_create_with_capacity(1, NULL)) == NULL ||
-			    apol_vector_append(reverse_v, t) < 0 || apol_vector_append(map->pseudo_to_mod, reverse_v) < 0) {
+			    apol_vector_append(reverse_v, (void *)t) < 0 || apol_vector_append(map->pseudo_to_mod, reverse_v) < 0) {
 				error = errno;
 				ERR(diff, "%s", strerror(error));
 				goto cleanup;
@@ -661,12 +609,12 @@ struct type_map_comp
  */
 static int type_map_primary_comp(const void *a, const void *b, void *data)
 {
-	qpol_type_t *ta = (qpol_type_t *) a;
-	qpol_type_t *tb = (qpol_type_t *) b;
+	const qpol_type_t *ta = a;
+	const qpol_type_t *tb = b;
 	struct type_map_comp *c = (struct type_map_comp *)data;
 	poldiff_t *diff = c->diff;
 	int dir = c->dir;
-	char *na, *nb;
+	const char *na, *nb;
 	if (dir == POLDIFF_POLICY_ORIG) {
 		if (qpol_type_get_name(diff->orig_qpol, ta, &na) < 0 || qpol_type_get_name(diff->mod_qpol, tb, &nb) < 0) {
 			return -1;
@@ -691,12 +639,12 @@ static int type_map_primary_comp(const void *a, const void *b, void *data)
  */
 static int type_map_prim_alias_comp(const void *a, const void *b, void *data)
 {
-	qpol_type_t *ta = (qpol_type_t *) a;
-	qpol_type_t *tb = (qpol_type_t *) b;
+	const qpol_type_t *ta = a;
+	const qpol_type_t *tb = b;
 	struct type_map_comp *c = (struct type_map_comp *)data;
 	poldiff_t *diff = c->diff;
 	int dir = c->dir;
-	char *prim, *alias;
+	const char *prim, *alias;
 	qpol_iterator_t *iter = NULL;
 	if (dir == POLDIFF_POLICY_ORIG) {
 		if (qpol_type_get_alias_iter(diff->orig_qpol, ta, &iter) < 0 || qpol_type_get_name(diff->mod_qpol, tb, &prim) < 0) {
@@ -802,7 +750,7 @@ static void type_remap_vector_dump(poldiff_t * diff)
 	v = poldiff_type_remap_get_entries(diff);
 	for (i = 0; i < apol_vector_get_size(v); i++) {
 		e = apol_vector_get_element(v, i);
-		printf("%d\t%s\t", i, poldiff_type_remap_entry_get_is_enabled(e) ? "en" : "dis");
+		printf("%zd\t%s\t", i, poldiff_type_remap_entry_get_is_enabled(e) ? "en" : "dis");
 		w = poldiff_type_remap_entry_get_original_types(diff, e);
 		for (j = 0; j < apol_vector_get_size(w); j++) {
 			name = apol_vector_get_element(w, j);
@@ -821,6 +769,29 @@ static void type_remap_vector_dump(poldiff_t * diff)
 #endif
 }
 
+static int type_map_entry_append_qtypes(poldiff_t * diff, poldiff_type_remap_entry_t * entry, const qpol_type_t * t,
+					const qpol_type_t * u)
+{
+	const char *name;
+	char *dup_name;
+	if (qpol_type_get_name(diff->orig_qpol, t, &name) < 0) {
+		return -1;
+	}
+	if ((dup_name = strdup(name)) == NULL || apol_vector_append(entry->orig_types, (void *)dup_name) < 0) {
+		free(dup_name);
+		return -1;
+	}
+
+	if (qpol_type_get_name(diff->mod_qpol, u, &name) < 0) {
+		return -1;
+	}
+	if ((dup_name = strdup(name)) == NULL || apol_vector_append(entry->mod_types, (void *)dup_name) < 0) {
+		free(dup_name);
+		return -1;
+	}
+	return 0;
+}
+
 int type_map_infer(poldiff_t * diff)
 {
 	apol_vector_t *ov = NULL, *mv = NULL;
@@ -853,8 +824,7 @@ int type_map_infer(poldiff_t * diff)
 		}
 		assert(!mod_done[j]);
 		u = (qpol_type_t *) apol_vector_get_element(mv, j);
-		if ((entry = poldiff_type_remap_entry_create(diff)) == NULL ||
-		    apol_vector_append(entry->orig_types, t) < 0 || apol_vector_append(entry->mod_types, u) < 0) {
+		if ((entry = poldiff_type_remap_entry_create(diff)) == NULL || type_map_entry_append_qtypes(diff, entry, t, u) < 0) {
 			error = errno;
 			ERR(diff, "%s", strerror(error));
 			goto cleanup;
@@ -884,8 +854,7 @@ int type_map_infer(poldiff_t * diff)
 		if (j >= num_mod) {
 			continue;
 		}
-		if ((entry = poldiff_type_remap_entry_create(diff)) == NULL ||
-		    apol_vector_append(entry->orig_types, t) < 0 || apol_vector_append(entry->mod_types, u) < 0) {
+		if ((entry = poldiff_type_remap_entry_create(diff)) == NULL || type_map_entry_append_qtypes(diff, entry, t, u) < 0) {
 			error = errno;
 			ERR(diff, "%s", strerror(error));
 			goto cleanup;
@@ -915,8 +884,7 @@ int type_map_infer(poldiff_t * diff)
 		if (i >= num_orig) {
 			continue;
 		}
-		if ((entry = poldiff_type_remap_entry_create(diff)) == NULL ||
-		    apol_vector_append(entry->orig_types, t) < 0 || apol_vector_append(entry->mod_types, u) < 0) {
+		if ((entry = poldiff_type_remap_entry_create(diff)) == NULL || type_map_entry_append_qtypes(diff, entry, t, u) < 0) {
 			error = errno;
 			ERR(diff, "%s", strerror(error));
 			goto cleanup;
@@ -946,8 +914,7 @@ int type_map_infer(poldiff_t * diff)
 		if (j >= num_mod) {
 			continue;
 		}
-		if ((entry = poldiff_type_remap_entry_create(diff)) == NULL ||
-		    apol_vector_append(entry->orig_types, t) < 0 || apol_vector_append(entry->mod_types, u) < 0) {
+		if ((entry = poldiff_type_remap_entry_create(diff)) == NULL || type_map_entry_append_qtypes(diff, entry, t, u) < 0) {
 			error = errno;
 			ERR(diff, "%s", strerror(error));
 			goto cleanup;
@@ -970,7 +937,7 @@ int type_map_infer(poldiff_t * diff)
 	return retval;
 }
 
-uint32_t type_map_lookup(poldiff_t * diff, qpol_type_t * type, int which_pol)
+uint32_t type_map_lookup(const poldiff_t * diff, const qpol_type_t * type, int which_pol)
 {
 	uint32_t val;
 	if (which_pol == POLDIFF_POLICY_ORIG) {
@@ -990,7 +957,7 @@ uint32_t type_map_lookup(poldiff_t * diff, qpol_type_t * type, int which_pol)
 	}
 }
 
-apol_vector_t *type_map_lookup_reverse(poldiff_t * diff, uint32_t val, int which_pol)
+const apol_vector_t *type_map_lookup_reverse(const poldiff_t * diff, uint32_t val, int which_pol)
 {
 	if (which_pol == POLDIFF_POLICY_ORIG) {
 		return apol_vector_get_element(diff->type_map->pseudo_to_orig, val - 1);
@@ -999,11 +966,11 @@ apol_vector_t *type_map_lookup_reverse(poldiff_t * diff, uint32_t val, int which
 	}
 }
 
-const char *type_map_get_name(poldiff_t * diff, const uint32_t pseudo_val, int pol)
+const char *type_map_get_name(const poldiff_t * diff, const uint32_t pseudo_val, int pol)
 {
-	apol_vector_t *v = NULL;
-	char *name = NULL;
-	qpol_type_t *t;
+	const apol_vector_t *v = NULL;
+	const char *name = NULL;
+	const qpol_type_t *t;
 
 	v = type_map_lookup_reverse(diff, pseudo_val, pol);
 	if (apol_vector_get_size(v) == 0) {
