@@ -150,11 +150,11 @@ static void policy_view_on_find_terules_click(GtkButton * button __attribute__ (
 	apol_avrule_query_t *query = apol_avrule_query_create();
 	apol_avrule_query_set_regex(policy, query, 1);
 	struct find_terules_datum run;
-	char *s;
+	const char *s;
 	gboolean only_direct;
 	apol_avrule_query_set_rules(policy, query, QPOL_RULE_ALLOW);
 	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pv->stype_check))) {
-		s = gtk_combo_box_get_active_text(GTK_COMBO_BOX(pv->stype_combo));
+		s = util_combo_box_get_active_text(GTK_COMBO_BOX(pv->stype_combo));
 		only_direct = gtk_toggle_button_get_active(pv->stype_direct);
 		if (strcmp(s, "") == 0) {
 			toplevel_ERR(pv->top, "No source type was selected.");
@@ -164,7 +164,7 @@ static void policy_view_on_find_terules_click(GtkButton * button __attribute__ (
 		apol_avrule_query_set_source_component(policy, query, APOL_QUERY_SYMBOL_IS_TYPE);
 	}
 	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pv->ttype_check))) {
-		s = gtk_combo_box_get_active_text(GTK_COMBO_BOX(pv->ttype_combo));
+		s = util_combo_box_get_active_text(GTK_COMBO_BOX(pv->ttype_combo));
 		only_direct = gtk_toggle_button_get_active(pv->ttype_direct);
 		if (strcmp(s, "") == 0) {
 			toplevel_ERR(pv->top, "No target type was selected.");
@@ -174,7 +174,7 @@ static void policy_view_on_find_terules_click(GtkButton * button __attribute__ (
 		apol_avrule_query_set_source_component(policy, query, APOL_QUERY_SYMBOL_IS_TYPE);
 	}
 	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pv->class_check))) {
-		s = gtk_combo_box_get_active_text(GTK_COMBO_BOX(pv->class_combo));
+		s = util_combo_box_get_active_text(GTK_COMBO_BOX(pv->class_combo));
 		if (strcmp(s, "") == 0) {
 			toplevel_ERR(pv->top, "No object class was selected.");
 			return;
@@ -236,8 +236,8 @@ static void policy_view_on_class_toggle(GtkToggleButton * toggle, gpointer user_
 }
 
 static gboolean policy_view_on_line_event(GtkTextTag * tag
-					  __attribute__ ((unused)), GObject * event_object, GdkEvent * event,
-					  const GtkTextIter * iter, gpointer user_data)
+					  __attribute__ ((unused)), GObject * event_object
+					  __attribute__ ((unused)), GdkEvent * event, const GtkTextIter * iter, gpointer user_data)
 {
 	policy_view_t *pv = (policy_view_t *) user_data;
 	GtkTextIter start, end;
@@ -471,34 +471,44 @@ static void policy_view_populate_combo_boxes(policy_view_t * pv)
 	if (policy != NULL) {
 		qpol_policy_t *qp = apol_policy_get_qpol(policy);
 		size_t i;
-		qpol_type_t *type;
-		qpol_class_t *obj_class;
-		char *s;
+		const qpol_type_t *type;
+		const qpol_class_t *obj_class;
+		const char *s;
 		GtkTreeIter iter;
 		apol_vector_t *v;
 		apol_type_get_by_query(policy, NULL, &v);
 		for (i = 0; i < apol_vector_get_size(v); i++) {
 			type = apol_vector_get_element(v, i);
 			qpol_type_get_name(qp, type, &s);
-			apol_vector_append(pv->type_list, s);
+			apol_vector_append(pv->type_list, (void *)s);
 		}
 		apol_vector_destroy(&v);
 		apol_vector_sort(pv->type_list, apol_str_strcmp, NULL);
 		for (i = 0; i < apol_vector_get_size(pv->type_list); i++) {
 			s = apol_vector_get_element(pv->type_list, i);
+#ifdef GTK_2_8
 			gtk_list_store_insert_with_values(pv->type_model, &iter, i, 0, s, -1);
+#else
+			gtk_list_store_insert(pv->type_model, &iter, i);
+			gtk_list_store_set(pv->type_model, &iter, 0, s, -1);
+#endif
 		}
 		apol_class_get_by_query(policy, NULL, &v);
 		for (i = 0; i < apol_vector_get_size(v); i++) {
 			obj_class = apol_vector_get_element(v, i);
 			qpol_class_get_name(qp, obj_class, &s);
-			apol_vector_append(pv->class_list, s);
+			apol_vector_append(pv->class_list, (void *)s);
 		}
 		apol_vector_destroy(&v);
 		apol_vector_sort(pv->class_list, apol_str_strcmp, NULL);
 		for (i = 0; i < apol_vector_get_size(pv->class_list); i++) {
 			s = apol_vector_get_element(pv->class_list, i);
+#ifdef GTK_2_8
 			gtk_list_store_insert_with_values(pv->class_model, &iter, i, 0, s, -1);
+#else
+			gtk_list_store_insert(pv->class_model, &iter, i);
+			gtk_list_store_set(pv->class_model, &iter, 0, s, -1);
+#endif
 		}
 	}
 }
@@ -518,7 +528,7 @@ void policy_view_find_terules(policy_view_t * pv, seaudit_message_t * message)
 {
 	seaudit_message_type_e type = SEAUDIT_MESSAGE_TYPE_INVALID;
 	void *data = NULL;
-	char *stype = "", *ttype = "", *obj_class = "";
+	const char *stype = "", *ttype = "", *obj_class = "";
 	size_t i;
 	assert(pv->type_list != NULL);
 	assert(pv->class_list != NULL);
